@@ -124,6 +124,30 @@ class MultiPartUploadError(ObjectError):
     """Thrown when a confirmation of an upload is rejected."""
 
 
+class MultiPartUploadAlreadyExistsError(MultiPartUploadError):
+    """Thrown when trying to create a multi-part upload for an object for which another
+    upload is already active."""
+
+    def __init__(self, bucket_id: str, object_id: str):
+        message = (
+            f"Failed to initiate a multi-part upload for object '{object_id}' in bucket"
+            + f" '{bucket_id}. Another upload is already ongoing for that file."
+        )
+        super().__init__(message)
+
+
+class MultipleActiveUploadsError(MultiPartUploadError):
+    """Thrown when multiple active multi-part uploads are detected for the same object."""
+
+    def __init__(self, bucket_id: str, object_id: str, upload_ids: list[str]):
+        message = (
+            f"Multiple active multi-part uploads were detected for object '{object_id}'"
+            + f" in bucket '{bucket_id}. Another upload is already ongoing for that"
+            + f" file. The IDs of the active uploads are: {','.join(upload_ids)}"
+        )
+        super().__init__(message)
+
+
 class MultiPartUploadNotFoundError(MultiPartUploadError):
     """Thrown when a upload with the specified upload, bucket, and object id was not found."""
 
@@ -135,7 +159,7 @@ class MultiPartUploadNotFoundError(MultiPartUploadError):
         details: Optional[str] = None,
     ):
         message = (
-            f"The upload with ID '{upload_id}' for object '{object_id}'"
+            f"The multi-part upload with ID '{upload_id}' for object '{object_id}'"
             + f" in bucket '{bucket_id} could not be found"
             + (f": {details}." if details else ".")
         )
@@ -153,13 +177,30 @@ class MultiPartUploadConfirmError(MultiPartUploadError):
         reason: Optional[str] = None,
     ):
         message = (
-            f"The confirmation of upload '{upload_id}' for object '{object_id}'"
-            + f" in bucket '{bucket_id} was rejected"
+            f"The confirmation of multi-part upload '{upload_id}' for object"
+            + f" '{object_id}' in bucket '{bucket_id} was rejected"
             + (f": {reason}." if reason else ".")
         )
         super().__init__(message)
 
 
+class MultiPartUploadAbortError(MultiPartUploadError):
+    """Thrown when failed to abort a multi-part upload."""
+
+    def __init__(
+        self,
+        upload_id: str,
+        bucket_id: str,
+        object_id: str,
+    ):
+        message = (
+            f"Failed to abort the multi-part upload '{upload_id}' for object"
+            + f" '{object_id}' in bucket '{bucket_id}'. An ongoing part upload might"
+            + " be a reason. Please complete all part upload and try to abort again."
+        )
+        super().__init__(message)
+
+
 def validate_bucket_id(bucket_id: str):
     """Check whether a bucket id follows the recommended naming pattern.
     This is roughly based on:
@@ -245,7 +286,7 @@ class ObjectStorageDao(DaoGenericBase):
     # by implementing `__enter__` and `__exit__`
     # methods:
 
-    def __enter__(self):
+    def __enter__(self) -> "ObjectStorageDao":
         """Setup storage connection/session."""
         ...
         return self
@@ -312,6 +353,17 @@ class ObjectStorageDao(DaoGenericBase):
         """
         raise NotImplementedError()
 
+    def abort_multipart_upload(
+        self,
+        upload_id: str,
+        bucket_id: str,
+        object_id: str,
+    ) -> None:
+        """Cancel a multipart upload with the specified ID. All uploaded content is
+        deleted.
+        """
+        raise NotImplementedError()
+
     # pylint: disable=too-many-arguments
     def complete_multipart_upload(
         self,
